import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { takeUntil, flatMap, tap, debounceTime, switchMap, filter } from 'rxjs/operators';

import * as _ from 'lodash';
import { AppStateService } from 'app/core/services/app.state.service';
import { NotificationService } from 'app/shared/services/notification.service';
import { environment } from 'environments/environment';
import { User } from 'app/core/models/user';
import { combineLatest, Subject } from 'rxjs';
import { DistinctHelper } from 'app/shared/helper/distinct.helper';
import { DegreeProgram } from 'app/features/modules/models/degree-program';
import { CourseService } from '../../services/course.service';
import { Course } from '../../models/course';
import { Term } from 'app/features/modules/models/term';
import { BackendError } from 'app/shared/models/backend-error';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { ComponentWithOptionalLifecycleHooks } from 'app/shared/route-reuse-life.cycle.directive';

export interface ProgramGroup {
  program: DegreeProgram;
  courses: Course[];
  description: string;
}

@Component({
  selector: 'ngx-courses',
  templateUrl: './courses.component.html',
  styleUrls: ['./courses.component.scss']
})
export class CoursesComponent extends BaseComponent implements OnInit, ComponentWithOptionalLifecycleHooks {
  loading: boolean = true;
  searching: boolean = false;
  programId: number;
  programGroups: ProgramGroup[] = [];
  principal: SubatoPrincipal;
  term: Term | null;
  error: BackendError;
  searchingError: BackendError;

  query: string;
  private queryChanged: Subject<string> = new Subject<string>();
  detached: boolean = false;

  constructor(
    private courseService: CourseService,
    private notificationService: NotificationService,
    private router: Router, private route: ActivatedRoute,
    private authService: AuthService,
    private appStateService: AppStateService
  ) {
    super();
  }

  onAttach() {
    this.detached = false;
  }

  onDetach() {
    this.detached = true;
  }

  ngOnInit(): void {
    combineLatest(this.authService.principal$, this.appStateService.term$)
      .pipe(
        takeUntil(this.unsubscribe),
        filter(() => !this.detached),
        flatMap(([principal, term]) => {
          this.principal = principal;
          this.term = term;
          this.loading = true;
          return this.courseService.getAll({ termId: term?.id, query: this.query });
        })
      ).subscribe(courses => {
        this.programGroups = this.getProgramGroups(courses);

        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });

    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        filter(() => !this.detached),
        debounceTime(500),
        tap(query => this.query = query),
        switchMap(query => {
          this.searching = true;
          return this.courseService.getAll({ termId: this.term?.id, query: query })
        })
      )
      .subscribe(courses => {
        this.searching = false;
        this.programGroups = this.getProgramGroups(courses);
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.searchingError = error;
          this.searching = false;
        } else {
          throw error;
        }
      });
  }

  filterCourses(query) {
    this.queryChanged.next(query);
  }

  getProgramGroups(courses: Course[]) {
    let programs = courses.map(course => course.modules).flat().map(m => m.program);
    let distinctPrograms = DistinctHelper.distinct(programs, (program) => program.id);
    let programGroups: ProgramGroup[] = distinctPrograms.map(program => {
      return {
        program: program,
        description: `PO ${program.poYear}`,
        courses: courses
          .filter(course => course.modules.some(m => m.program.id == program.id))
          .sort((a, b) => a.moduleForProgram(program).scheduledTerm - b.moduleForProgram(program).scheduledTerm)
      }
    })
      .sort((a, b) => a.program.dual - b.program.dual || a.program.name.localeCompare(b.program.name));

    var coursesWithoutModules = courses.filter(c => c.modules.length == 0);
    if (coursesWithoutModules.length != 0) {
      programGroups.splice(0, 0, {
        program: null,
        courses: coursesWithoutModules,
        description: 'Kurse ohne Bezug zu einem Studiengang'
      });
    }

    return programGroups;
  }
}
