import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { HttpCacheManager } from "@ngneat/cashew";
import { User } from "app/core/models/user";
import { ParticipantAttendance } from "app/features/attendances/dto/participant-attendance.dto";
import { BackendError } from "app/shared/models/backend-error";
import { DataQueryParams } from "app/shared/models/data-query-params";
import { Page } from "app/shared/models/page";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { CourseUnit } from "../models/course-unit";
import { CourseUnitEvent } from "../models/course-unit-event";

@Injectable({
    providedIn: 'root'
})
export class CourseUnitService extends BaseService {
    constructor(private http: HttpClient, private cacheManager: HttpCacheManager) {
        super();
    }

    getById(id: number): Observable<CourseUnit> {
        return this.http.get<CourseUnit>(`${environment.backendUrl}/course_unit/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(course => new CourseUnit(course))
        )
    }

    getEvents(id: number): Observable<CourseUnitEvent[]> {
        return this.http.get<CourseUnitEvent[]>(`${environment.backendUrl}/course_unit/${id}/events`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(events => events.map(event => new CourseUnitEvent(event)))
        )
    }

    getParticipants(id: number): Observable<User[]> {
        return this.http.get<User[]>(`${environment.backendUrl}/course_unit/${id}/participants`, {
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(users => users.map(u => new User(u)))
        )
    }

    addParticipant(id: number, userId: number): Observable<void> {
        return this.http.put<void>(`${environment.backendUrl}/course_unit/${id}/participants/${userId}`, {}).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(() => this.cacheManager.clear())
        );
    }

    removeParticipant(id: number, userId: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/course_unit/${id}/participants/${userId}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(() => this.cacheManager.clear())
        );
    }

    getAttendancesForUserAndUnit(courseUnitId: number, userId: number): Observable<ParticipantAttendance> {
        let url = `${environment.backendUrl}/course_unit/${courseUnitId}/attendance/${userId}`;
        return this.http.get<ParticipantAttendance>(url).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(dto => new ParticipantAttendance(dto))
        );
    }
}
