import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { User } from 'app/core/models/user';
import { AuthService } from 'app/core/services/auth.service';
import { ParticipantAttendance } from 'app/features/attendances/dto/participant-attendance.dto';
import { AttendanceService } from 'app/features/attendances/services/attendance.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin, of } from 'rxjs';
import { takeUntil, flatMap, take, map, filter } from 'rxjs/operators';
import { CourseUnit } from '../../../courses/models/course-unit';
import { CourseUnitService } from '../../../courses/services/course-unit.service';
import { BackendError } from 'app/shared/models/backend-error';
import { ExerciseService } from 'app/features/exercises/services/exercise.service';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { ComponentWithOptionalLifecycleHooks } from 'app/shared/route-reuse-life.cycle.directive';
import { CourseService } from 'app/features/courses/services/course.service';
import { CourseResult } from 'app/features/courses/models/course-result';
import { ParamHelper } from 'app/shared/helper/param.helper';

@Component({
  selector: 'ngx-course-unit',
  templateUrl: './course-unit.component.html',
  styleUrls: ['./course-unit.component.scss']
})
export class CourseUnitComponent extends BaseComponent implements OnInit, ComponentWithOptionalLifecycleHooks {
  selectedUser: User;
  principal: SubatoPrincipal;
  loading: boolean = false;
  loadingProgress: boolean = false;
  courseUnit: CourseUnit;
  participantAttendance: ParticipantAttendance;
  courseResult: CourseResult;
  hideInvisible = true;

  loadingProgressError: BackendError;
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Veranstaltung konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  detached: boolean = false;


  constructor(private exerciseService: ExerciseService,
    private appStateService: AppStateService,
    private courseService: CourseService,
    private courseUnitService: CourseUnitService,
    private attendanceService: AttendanceService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private route: ActivatedRoute,
    private router: Router) {
    super();
  }

  public onAttach() {
    this.detached = false;
    this.appStateService.setCourse(this.courseUnit?.course);
  }

  public onDetach() {
    this.detached = true;
  }
  
  buildBreadcrumbs(unit: CourseUnit) {
    this.breadcrumbs = [
      {
        id: 'courseUnits',
        params: {
          course_id: unit.course.id
        }
      },
      {
        id: 'courseUnit',
        params: {
          unit_id: unit.id
        },
        ctx: unit
      },
    ];
  }

  ngOnInit(): void {
    this.loading = true;
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        filter(() => !this.detached),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let unitId = ParamHelper.parseIdOrThrow(params, "unit_id");
          return this.courseUnitService.getById(unitId);
        })
      )
      .subscribe(courseUnit => {
        this.courseUnit = courseUnit;
        this.appStateService.setCourse(courseUnit.course);
        this.buildBreadcrumbs(courseUnit);
        this.loading = false;
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  onHideInvisibleChanged(hideInvisible: boolean) {
    this.loadingProgress = true;
    this.hideInvisible = hideInvisible;
    this.courseService.getProgressForUser(this.courseUnit.course.id, this.selectedUser.id, this.hideInvisible)
      .pipe(take(1))
      .subscribe(courseResult => {
        this.courseResult = courseResult;
        this.loadingProgress = false;
      }, (error) => {
        this.loadingProgress = false;
        this.loadingProgressError = error;
      });
  }

  selectUser(user: User) {
    this.selectedUser = user;
    this.loadingProgress = true;

    return forkJoin(
      this.courseUnitService.getAttendancesForUserAndUnit(this.courseUnit.id, user.id), 
      this.courseService.getProgressForUser(this.courseUnit.course.id, user.id, this.hideInvisible)
    )
    .pipe(
      takeUntil(this.unsubscribe),
    )
    .subscribe(([participantAttendance, courseResult]) => {
      this.participantAttendance = participantAttendance;
      this.courseResult = courseResult;
      this.loadingProgress = false;
    }, (error) => {
      this.loadingProgress = false;
      this.loadingProgressError = error;
    });
  }
}
