package de.hsrm.sls.subato.exercises.exercise;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.solutions.solution.SolutionService;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceService;
import de.hsrm.sls.subato.shared.DangerousActionException;
import de.hsrm.sls.subato.tasks.task.TaskRepository;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ExerciseService {
    @Autowired
    private ExerciseRepository exerciseRepository;

    @Autowired
    private TaskRepository taskRepository;

    @Autowired
    private TaskInstanceService taskInstanceService;

    @Autowired
    private PolicyEvaluator evaluator;

    @Autowired
    private SolutionService solutionService;

    public Exercise findById(long id) {
        return findById(id, true);
    }

    public Exercise findById(long id, boolean raiseException) {
        var exercise = exerciseRepository.findById(id);

        if(raiseException) {
            return exercise.orElseThrow(() -> new ExerciseNotFoundException(id));
        } else {
            return exercise.orElse(null);
        }
    }

    private void updateTaskInstances(Exercise exercise, CreateExerciseDto dto, boolean confirmDangerousActions) {
        var toRemove = exercise.getTaskInstances().stream()
                .filter(ti -> !ti.getTask().isArchived())
                .filter(ti -> !dto.getTaskInstances().stream().anyMatch(tiDto -> ti.getTask().getId()  == tiDto.getTaskId()))
                .collect(Collectors.toList());

        var taskInstancesWithSolutions = new HashMap<TaskInstance, Long>();
        for(var taskInstance : toRemove) {
            var solutionCount = solutionService.getSolutionCountForTaskInstance(taskInstance);
            if(solutionCount != 0) {
                taskInstancesWithSolutions.put(taskInstance, solutionCount);
            }
        }

        if(taskInstancesWithSolutions.size() != 0 && !confirmDangerousActions) {
            var infoString = taskInstancesWithSolutions.entrySet().stream().map(e -> "%s (%s Lösungen)".formatted(e.getKey().getTask().getName(), e.getValue()))
                    .collect(Collectors.joining(", "));
            throw new DangerousActionException(
                    "Einige Aufgaben haben Lösungen, die beim Speichern gelöscht werden würden: %s".formatted(infoString),
                    "Eventuell wäre es sinnvoller die Aufgaben nicht zu löschen, sondern stattdessen als optional zu markieren"
            );
        }

        exercise.getTaskInstances().removeAll(toRemove);
        for(var taskInstance : toRemove) {
            solutionService.deleteByTaskInstance(taskInstance);
        }

        for(var taskInstanceDto : dto.getTaskInstances()) {
            var taskInstance = exercise.getTaskInstances().stream()
                    .filter(ti -> ti.getTask().getId() == taskInstanceDto.getTaskId())
                    .findFirst()
                    .orElse(null);

            if(taskInstance == null) {
                taskInstance = new TaskInstance();
                exercise.getTaskInstances().add(taskInstance);

                taskInstance.setExercise(exercise);
                var task = taskRepository.findById(taskInstanceDto.getTaskId()).orElse(null);
                if(task == null) {
                    throw new RuntimeException("Task not found with id %s".formatted(taskInstanceDto.getTaskId()));
                }
                taskInstance.setTask(task);
            }

            taskInstance.setReachablePoints(taskInstanceDto.getReachablePoints());
            taskInstance.setOrderNumber(taskInstanceDto.getOrderNumber());
        }
    }

    private void setExerciseFields(Exercise exercise, CreateExerciseDto dto, boolean confirmDangerousActions) {
        exercise.setDescription(dto.getDescription());
        exercise.setSummary(dto.getSummary());
        exercise.setName(dto.getName());
        exercise.setAssessmentMode(dto.getAssessmentMode());
        exercise.setDueTo(dto.getDueTo());
        exercise.setVisibleFrom(dto.getVisibleFrom());
        updateTaskInstances(exercise, dto, confirmDangerousActions);
    }

    private void validateExercise(Exercise exercise) {
        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(exercise);
        if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
        }
    }

    public Exercise create(Course course, CreateExerciseDto dto) {
        var exercise = new Exercise();
        exercise.setCourse(course);
        setExerciseFields(exercise, dto, false);

        validateExercise(exercise);
        return exerciseRepository.save(exercise);
    }

    /**
     * updates the exercise with the given field values
     * @param exercise to update
     * @param dto field values to update
     * @param confirmDangerousActions if this update action was explicitly confirmed by the user.
     *                                required if the action is dangerous and needs to be confirmed again.
     *                                the update is dangerous if task instances are removed that have linked solutions
     *                                because these solutions would be deleted
     * @return updated exercise
     * @throws DangerousActionException if solutions will be removed while updating and {@code confirmDangerousActions} is false
     */
    @Transactional
    Exercise update(Exercise exercise, CreateExerciseDto dto, boolean confirmDangerousActions) {
        setExerciseFields(exercise, dto, confirmDangerousActions);
        validateExercise(exercise);
        return exerciseRepository.save(exercise);
    }

    public List<Exercise> findAll(ExerciseFilter filter) {
        return findAll(filter, null);
    }

    public List<Exercise> findAll(ExerciseFilter filter, String permission) {
        Specification<Exercise> spec = filter.toSpecification();

        if(permission != null) {
            var authSpec = evaluator.checkAsSpec(CheckPolicyRequest.fromPrincipal(null, permission));
            spec = spec.and(authSpec);
        }

        var exercises = exerciseRepository.findAll(spec);
        exercises.sort(Comparator.comparing(Exercise::getId));
        return exercises;
    }

    /**
     * deletes an exercise, including all linked task instances, solutions and submissions
     */
    public void delete(Exercise exercise) {
        this.exerciseRepository.delete(exercise);
    }


}
