package de.hsrm.sls.subato.exercises.submission;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.Optional;

import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Übungsverwaltung")
public class SubmissionController {
    @Autowired
    private SubmissionMapper submissionMapper;

    @Autowired
    private SubmissionService submissionService;

    @Autowired
    private PrincipalService principalService;

    @Autowired
    private PolicyEvaluator policy;

    @GetMapping("/submission/{id}")
    SubmissionDetailDto getOne(@PathVariable Long id) {
        try {
            var submission = submissionService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(submission, ExercisesAction.SUBMISSION_VIEW));

            return submissionMapper.mapDetail(submission);
        } catch(SubmissionNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "Die Abgabe existiert nicht");
        }
    }

    @PostMapping("/submission/{id}/passed")
    @Operation(summary = "Über Abgabe entscheiden: Bestanden, nicht bestanden oder keine Entscheidung")
    SubmissionDetailDto updatePassed(@PathVariable Long id, @RequestBody UpdatePassedDto dto) {
        try {
            var submission = submissionService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(submission, ExercisesAction.SUBMISSION_UPDATE));

            submission = submissionService.updatePassed(submission, dto, principalService.getPrincipal());
            return submissionMapper.mapDetail(submission);
        } catch(SubmissionNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "Die Abgabe existiert nicht");
        }
    }

    @PostMapping("/submission/{id}/points")
    @Operation(summary = "Erreichte Punktzahl einer Aufgabeninstanz innerhalb der Abgabe (Aufgabe in Übungsblatt) setzen. Nur möglich falls Aufgabeninstanz eine konfigurierte Punktzahl hat und zugeordnete Übung bepunktet ist.")
    SubmissionDetailDto updatePoints(@PathVariable Long id, @RequestBody UpdatePointsDto dto) {
        try {
            var submission = submissionService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(submission, ExercisesAction.SUBMISSION_UPDATE));

            submission = submissionService.updatePoints(submission, dto);
            return submissionMapper.mapDetail(submission);
        } catch(SubmissionNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "Die Abgabe existiert nicht");
        }
    }

    @GetMapping("/submission")
    @PreAuthorize("isAuthenticated()")
    @Operation(summary = "Alle sichtbaren Abgaben im Kontext des angemeldeten Benutzers")
    List<SubmissionSummaryDto> getAll(@RequestParam Optional<Long> exerciseId, @RequestParam Optional<Long> userId, @RequestParam Optional<Long> unitId) {
        var filter = new SubmissionFilter()
                .exerciseId(exerciseId.orElse(null))
                .unitId(unitId.orElse(null))
                .submittedById(userId.orElse(null));
        var submissions = submissionService.findAll(filter, ExercisesAction.SUBMISSION_VIEW);

        return submissions.stream().map(submission -> submissionMapper.mapSummary(submission)).toList();
    }
}
