package de.hsrm.sls.subato.solutions.submit;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.exercises.exercise.ExerciseService;
import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.solutions.solution.Client;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.stef.spec.SupportedArchiveExtensions;
import de.hsrm.sls.subato.tasks.stef.spec.FileType;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.TaskArchivedException;
import de.hsrm.sls.subato.tasks.task.TaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.ArrayList;

/**
 * responsible for redirecting submitted solutions to the correct handlers for further processing
 */
@Service
public class SubmitGate {
    @Autowired
    private TaskService taskService;

    @Autowired
    private ExerciseService exerciseService;

    @Autowired
    private ExerciseSubmitHandler exerciseSubmitHandler;

    @Autowired
    private TaskSubmitHandler taskSubmitHandler;

    @Autowired
    private PrincipalService principalService;

    /**
     * forwards the request to the correct handler, which determines how to process this solution
     */
    private Solution forward(SubmitSolutionRequest request) {
        SubmitHandler handler = request.exercise() != null ? exerciseSubmitHandler : taskSubmitHandler;
        return handler.handle(request, principalService.getPrincipal(), LocalDateTime.now());
    }

    /**
     * wraps the controller input (solution specified inline as string, e.g. through a text editor) into a submit request
     * and forwards it for further processing to the correct handler
     * @throws InvalidModeException if the submission mode is not supported by the task
     * @throws de.hsrm.sls.subato.tasks.task.TaskNotFoundException
     * @throws de.hsrm.sls.subato.tasks.task.TaskArchivedException
     * @throws de.hsrm.sls.subato.exercises.exercise.ExerciseNotFoundException
     * @throws de.hsrm.sls.subato.solutions.evaluation.EvaluationFailedException
     * @throws de.hsrm.sls.subato.solutions.evaluation.InvalidSolutionException
     */
    public Solution submit(SubmitInlineSolution dto) {
        var task = taskService.findById(dto.getTaskId());
        var exercise = dto.getExerciseId() != null ? exerciseService.findById(dto.getExerciseId()) : null;

        if(task.isArchived()) {
            throw new TaskArchivedException();
        }

        if(!task.isSupportsTextSubmissionMode()) {
            throw new InvalidModeException();
        }

        var taskFiles = taskService.getFiles(task);
        var templateTaskFile = taskFiles.stream().filter(f -> f.getType() == FileType.TEMPLATE).findFirst().orElse(null);
        var content = FileContent.fromString(templateTaskFile != null ? templateTaskFile.getFile().getName() : "%s.%s".formatted(FileRef.DEFAULT_FILE_NAME, FileRef.DEFAULT_FILE_EXT), dto.getContent());

        var request = new SubmitSolutionRequest(exercise, task, dto.isLateSubmit(), dto.getComment(), Client.Web, content);
        return forward(request);
    }

    /**
     * wraps the controller input (solution specified as file) into a submit request
     * and forwards it for further processing to the correct handler
     * @throws InvalidModeException if the submission mode is not supported by the task
     * @throws de.hsrm.sls.subato.tasks.task.TaskNotFoundException
     * @throws de.hsrm.sls.subato.tasks.task.TaskArchivedException
     * @throws de.hsrm.sls.subato.exercises.exercise.ExerciseNotFoundException
     * @throws de.hsrm.sls.subato.solutions.evaluation.EvaluationFailedException
     * @throws de.hsrm.sls.subato.solutions.evaluation.InvalidSolutionException
     * @throws ExtensionNotAllowedException
     */
    public Solution submit(MultipartFile file, SubmitFileSolutionDto dto) {
        var task = taskService.findById(dto.getTaskId());
        var exercise = dto.getExerciseId() != null ? exerciseService.findById(dto.getExerciseId()) : null;

        if(task.isArchived()) {
            throw new TaskArchivedException();
        }

        if(!task.isSupportsFileSubmissionMode()) {
            throw new InvalidModeException();
        }

        var content = FileContent.fromMultipartFile(file);
        validateExtensionConstraints(content, task);

        var request = new SubmitSolutionRequest(exercise, task, dto.isLateSubmit(), dto.getComment(), dto.getSubmittedWith(), content);
        return forward(request);
    }

    /**
     * checks if the submitted solution fulfills the filetype extension constraints specified in the task
     * @throws ExtensionNotAllowedException if it does not
     */
    private void validateExtensionConstraints(FileContent fileContent, Task task) {
        var allowedExtensions = new ArrayList<String>();

        var fileSubmissionMode = task.getFileSubmissionMode();
        allowedExtensions.addAll(fileSubmissionMode.getExtensions());

        if(task.isAllowArchives()) {
            var supportedArchiveExtensions = SupportedArchiveExtensions.getSupportedExtensions();
            allowedExtensions.addAll(supportedArchiveExtensions);
        }

        for(var extension : allowedExtensions) {
            if(fileContent.getFileName().endsWith(extension)) {
                return;
            }
        }

        throw new ExtensionNotAllowedException(fileContent.getFileName(), allowedExtensions);
    }
}
